package at.ac.uibk.gitsearch.service.criteria;

import at.ac.uibk.gitsearch.domain.enumeration.CheckFrequency;
import java.io.Serializable;
import java.util.Objects;
import tech.jhipster.service.Criteria;
import tech.jhipster.service.filter.Filter;
import tech.jhipster.service.filter.LongFilter;
import tech.jhipster.service.filter.StringFilter;

/**
 * Criteria class for the {@link at.ac.uibk.gitsearch.domain.UserWatchList} entity. This class is used
 * in {@link at.ac.uibk.gitsearch.web.rest.UserWatchListResource} to receive all the possible filtering options from
 * the Http GET request parameters.
 * For example the following could be a valid request:
 * {@code /user-watch-lists?id.greaterThan=5&attr1.contains=something&attr2.specified=false}
 * As Spring is unable to properly convert the types, unless specific {@link Filter} class are used, we need to use
 * fix type specific filters.
 */
@SuppressWarnings(
    {
        "PMD.GodClass",
        "PMD.AvoidFieldNameMatchingMethodName",
        "PMD.FieldDeclarationsShouldBeAtStartOfClass",
        "PMD.ShortMethodName",
        "PMD.ConfusingTernary",
    }
)
public class UserWatchListCriteria implements Serializable, Criteria {

    /**
     * Class for filtering CheckFrequency
     */
    public static class CheckFrequencyFilter extends Filter<CheckFrequency> {

        private static final long serialVersionUID = 1L;

        public CheckFrequencyFilter() {
            super();
        }

        public CheckFrequencyFilter(CheckFrequencyFilter filter) {
            super(filter);
        }

        @Override
        public CheckFrequencyFilter copy() {
            return new CheckFrequencyFilter(this);
        }
    }

    private static final long serialVersionUID = 1L;

    private LongFilter id;

    private StringFilter name;

    private CheckFrequencyFilter checkFrequency;

    private LongFilter userId;

    private LongFilter watchListEntryId;

    private Boolean distinct;

    public UserWatchListCriteria() {}

    public UserWatchListCriteria(UserWatchListCriteria other) {
        this.id = other.id == null ? null : other.id.copy();
        this.name = other.name == null ? null : other.name.copy();
        this.checkFrequency = other.checkFrequency == null ? null : other.checkFrequency.copy();
        this.userId = other.userId == null ? null : other.userId.copy();
        this.watchListEntryId = other.watchListEntryId == null ? null : other.watchListEntryId.copy();
        this.distinct = other.distinct;
    }

    @Override
    public UserWatchListCriteria copy() {
        return new UserWatchListCriteria(this);
    }

    public LongFilter getId() {
        return id;
    }

    public LongFilter id() {
        if (id == null) {
            id = new LongFilter();
        }
        return id;
    }

    public void setId(LongFilter id) {
        this.id = id;
    }

    public StringFilter getName() {
        return name;
    }

    public StringFilter name() {
        if (name == null) {
            name = new StringFilter();
        }
        return name;
    }

    public void setName(StringFilter name) {
        this.name = name;
    }

    public CheckFrequencyFilter getCheckFrequency() {
        return checkFrequency;
    }

    public CheckFrequencyFilter checkFrequency() {
        if (checkFrequency == null) {
            checkFrequency = new CheckFrequencyFilter();
        }
        return checkFrequency;
    }

    public void setCheckFrequency(CheckFrequencyFilter checkFrequency) {
        this.checkFrequency = checkFrequency;
    }

    public LongFilter getUserId() {
        return userId;
    }

    public LongFilter userId() {
        if (userId == null) {
            userId = new LongFilter();
        }
        return userId;
    }

    public void setUserId(LongFilter userId) {
        this.userId = userId;
    }

    public LongFilter getWatchListEntryId() {
        return watchListEntryId;
    }

    public LongFilter watchListEntryId() {
        if (watchListEntryId == null) {
            watchListEntryId = new LongFilter();
        }
        return watchListEntryId;
    }

    public void setWatchListEntryId(LongFilter watchListEntryId) {
        this.watchListEntryId = watchListEntryId;
    }

    public Boolean getDistinct() {
        return distinct;
    }

    public void setDistinct(Boolean distinct) {
        this.distinct = distinct;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        final UserWatchListCriteria that = (UserWatchListCriteria) o;
        return (
            Objects.equals(id, that.id) &&
            Objects.equals(name, that.name) &&
            Objects.equals(checkFrequency, that.checkFrequency) &&
            Objects.equals(userId, that.userId) &&
            Objects.equals(watchListEntryId, that.watchListEntryId) &&
            Objects.equals(distinct, that.distinct)
        );
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name, checkFrequency, userId, watchListEntryId, distinct);
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "UserWatchListCriteria{" +
            (id != null ? "id=" + id + ", " : "") +
            (name != null ? "name=" + name + ", " : "") +
            (checkFrequency != null ? "checkFrequency=" + checkFrequency + ", " : "") +
            (userId != null ? "userId=" + userId + ", " : "") +
            (watchListEntryId != null ? "watchListEntryId=" + watchListEntryId + ", " : "") +
            (distinct != null ? "distinct=" + distinct + ", " : "") +
            "}";
    }
}
