import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable, Subject } from 'rxjs';

import { ApplicationConfigService } from 'app/core/config/application-config.service';

export class DeploymentInfo {
  branch = '';
  commitId = '';
  deploymentDate = '';
}
@Injectable({ providedIn: 'root' })
export class ApplicationInfoService {
  private cachedDeploymentInfo: DeploymentInfo;
  private inLoading = false;

  private deploymentInfoBroker = new Subject<DeploymentInfo>();

  constructor(private http: HttpClient, private applicationConfigService: ApplicationConfigService) {
    this.cachedDeploymentInfo = {} as DeploymentInfo;
  }

  public loadOerLink(): Observable<string> {
    return this.http.get(this.applicationConfigService.getEndpointFor('api/applicationInfo/oerLinkInfo'), { responseType: 'text' });
  }

  public getDeploymentInfo(): DeploymentInfo {
    if (!this.cachedDeploymentInfo.branch && !this.inLoading) {
      this.inLoading = true;
      this.loadDeploymentInfo().subscribe(res => {
        this.cachedDeploymentInfo = res;
        this.inLoading = false;
        this.deploymentInfoBroker.next(this.cachedDeploymentInfo);
      });
    }
    return this.cachedDeploymentInfo;
  }

  onDeploymentInfoChanged(): Observable<DeploymentInfo> {
    return this.deploymentInfoBroker;
  }

  private loadDeploymentInfo(): Observable<DeploymentInfo> {
    return this.http.get<DeploymentInfo>(SERVER_API_URL + 'api/applicationInfo/deploymentInfo');
  }
}
