import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { CloudData, CloudOptions } from 'angular-tag-cloud-module';
import { AutoCompletionEntry, SearchService } from 'app/search/service/search-service';
import Color from 'colorjs.io';

@Component({
  selector: 'jhi-teaser-content',
  templateUrl: './teaserContent.component.html',
  styleUrls: ['./teaserContent.component.scss'],
})
export class TeaserContentComponent implements OnInit {
  public keywords: Array<AutoCompletionEntry> = new Array<AutoCompletionEntry>();
  public contributors: Array<AutoCompletionEntry> = new Array<AutoCompletionEntry>();
  public programmingLanguages: Array<AutoCompletionEntry> = new Array<AutoCompletionEntry>();

  readonly options: CloudOptions = {
    overflow: false,
    zoomOnHover: { scale: 1.1, transitionTime: 0.5 },
    height: 350,
    width: 1,
    realignOnResize: true,
  };
  public keywordCloudData: CloudData[] = [];
  public contributorCloudData: CloudData[] = [];
  public programmingLanguageCloudData: CloudData[] = [];

  constructor(private searchService: SearchService, private router: Router) {}

  ngOnInit(): void {
    this.searchService.getKeywordsAutoComplete('', 20).subscribe({
      next: (data: Array<AutoCompletionEntry>) => {
        this.keywords = data;
        this.keywordCloudData = [];
        this.keywords.forEach(kw =>
          this.keywordCloudData.push({
            text: kw.target.toString(),
            weight: Math.min(kw.hitCount.valueOf(), 10),
            color: this.randomColor('#ff7777'),
          })
        );
      },
      error: () => {
        alert('Initialization of keywords failed');
      },
    });

    this.searchService.getProgrammingLanguageAutoComplete('').subscribe({
      next: (data: Array<AutoCompletionEntry>) => {
        this.programmingLanguages = data;
        this.programmingLanguageCloudData = [];
        this.programmingLanguages.forEach(pl =>
          this.programmingLanguageCloudData.push({
            text: pl.target.toString(),
            weight: Math.min(pl.hitCount.valueOf(), 10),
            color: this.randomColor('#7777ff'),
          })
        );
      },
      error: () => {
        alert('Initialization of programming languages failed');
      },
    });
    this.searchService.getContributorCreatorAutoComplete('', 10).subscribe({
      next: (data: Array<AutoCompletionEntry>) => {
        this.contributors = data;
        this.contributorCloudData = [];
        this.contributors.forEach(pl =>
          this.contributorCloudData.push({
            text: pl.target.toString(),
            weight: Math.min(pl.hitCount.valueOf(), 10),
            color: this.randomColor('#77ff77'),
          })
        );
      },
      error: () => {
        alert('Initialization of contributors failed');
      },
    });
  }

  onKeywordClick(event: CloudData): void {
    this.clickKeyword(event.text);
  }

  onProgrammingLanguageClick(event: CloudData): void {
    this.clickLanguage(event.text);
  }

  onContributorClick(event: CloudData): void {
    this.clickContributor(event.text);
  }

  clickLanguage(programmingLanguage: string): void {
    this.router.navigate(['/search'], { queryParams: { pl: programmingLanguage } });
  }

  clickContributor(contributor: string): void {
    this.router.navigate(['/search'], { queryParams: { a: contributor } });
  }

  clickKeyword(keyWord: string): void {
    this.router.navigate(['/search'], { queryParams: { kw: keyWord } });
  }

  private randomColor(mainColorHex: string): string {
    try {
      const main = new Color(mainColorHex);
      const deviation = new Color('#FFFFFF');

      const redMax = Math.min(main.get('red') * 2, 1.0);
      const greenMax = Math.min(main.get('green') * 2, 1.0);
      const blueMax = Math.min(main.get('blue') * 2, 1.0);

      const red = Math.max(redMax - 2 * deviation.get('red') * Math.random(), 0);
      const green = Math.max(greenMax - 2 * deviation.get('green') * Math.random(), 0);
      const blue = Math.max(blueMax - 2 * deviation.get('blue') * Math.random(), 0);

      const hexString = new Color('srgb', [red, green, blue]).toString({ format: 'hex' });
      return hexString;
      /*
      const upperBound = new Color(
        Math.min(main.getRed() + deviation.getRed(), 255),
        Math.min(main.getGreen() + deviation.getGreen(), 255),
        Math.min(main.getBlue() + deviation.getBlue(), 255)
      );
      const target2 = 
      const target = new Color(
        Math.max(Math.round(upperBound.getRed() - 2 * deviation.getRed() * Math.random()), 0),
        Math.max(Math.round(upperBound.getGreen() - 2 * deviation.getGreen() * Math.random()), 0),
        Math.max(Math.round(upperBound.getBlue() - 2 * deviation.getBlue() * Math.random()), 0)
      );
      return target.getHex();
      */
    } catch (e) {
      return mainColorHex;
    }
  }
}
