package at.ac.uibk.gitsearch.web.rest;

import static org.junit.Assert.fail;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.service.gitlab_events.GitlabEventServiceIT;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.nio.charset.Charset;
import java.util.stream.Stream;
import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.Named;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link LikesResource} REST controller.
 */
@SpringBootTest(classes = GitsearchApp.class)
@ExtendWith(MockitoExtension.class)
@AutoConfigureMockMvc
//@WithMockUser(value = "user", authorities = { "sharing", AuthoritiesConstants.ADMIN })
class GitlabResourceIT {

    @Autowired
    private MockMvc gitLabEventMockMvc;

    public static Stream<Object> allTestSystemHooks() throws IOException {
        return GitlabEventServiceIT.allTestSystemHooks().map(name -> Arguments.of(Named.of(name, getTestSystemHookEventAsString(name))));
    }

    private static String getTestSystemHookEventAsString(final String testEvent) {
        InputStream collectionCommit = GitlabEventServiceIT.class.getResourceAsStream(testEvent);
        StringWriter sw = new StringWriter();
        try {
            IOUtils.copy(collectionCommit, sw, Charset.forName("UTF-8"));
        } catch (IOException e) {
            fail("Cannot parse " + testEvent);
        }
        return sw.toString();
    }

    @ParameterizedTest
    @MethodSource(value = "allTestSystemHooks")
    @Transactional
    void testVariousEvents(String systemHookEvent) throws Exception {
        // Create the Likes
        gitLabEventMockMvc
            .perform(
                post("/api/gitlab/eventListener").with(csrf().asHeader()).contentType(MediaType.APPLICATION_JSON).content(systemHookEvent)
            )
            .andExpect(status().isOk());
    }
}
